@@ -35,7 +35,10 @@ class ViewBuilder(base.BaseViewBuilder):
     def _get_links(self, vnf_instance):
         links = {
             "self": {
-                "href": '/vnflcm/v1/vnf_instances/%s' % vnf_instance.id
+                "href":
+                '{endpoint}/vnflcm/v1/vnf_instances/{id}'.format(
+                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                    id=vnf_instance.id)
             }
         }
 
@@ -43,8 +46,10 @@ class ViewBuilder(base.BaseViewBuilder):
                 fields.VnfInstanceState.NOT_INSTANTIATED):
             instantiate_link = {
                 "instantiate": {
-                    "href": '/vnflcm/v1/vnf_instances/%s/instantiate'
-                            % vnf_instance.id
+                    "href":
+                    '{endpoint}/vnflcm/v1/vnf_instances/{id}/instantiate'
+                    .format(endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                        id=vnf_instance.id)
                 }
             }
 
@@ -54,12 +59,22 @@ class ViewBuilder(base.BaseViewBuilder):
                 fields.VnfInstanceState.INSTANTIATED):
             instantiated_state_links = {
                 "terminate": {
-                    "href": '/vnflcm/v1/vnf_instances/%s/terminate'
-                            % vnf_instance.id
+                    "href":
+                    '{endpoint}/vnflcm/v1/vnf_instances/{id}/terminate'
+                    .format(endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                        id=vnf_instance.id)
                 },
                 "heal": {
-                    "href": '/vnflcm/v1/vnf_instances/%s/heal'
-                            % vnf_instance.id
+                    "href":
+                    '{endpoint}/vnflcm/v1/vnf_instances/{id}/heal'
+                    .format(endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                        id=vnf_instance.id)
+                },
+                "changeExtConn": {
+                    "href":
+                    '{endpoint}/vnflcm/v1/vnf_instances/{id}/change_ext_conn'
+                    .format(endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                    id=vnf_instance.id)
                 }
             }
 
@@ -89,28 +104,45 @@ class ViewBuilder(base.BaseViewBuilder):
 
         return vim_connections
 
+    # TODO(esto-aln): This method will be transferred to
+    # tacker/api/views/vnf_lcm_op_occs.py in the future
     def _get_lcm_op_occs_links(self, vnf_lcm_op_occs):
         _links = {
             "self": {
-                "href": '%(endpoint)s/vnflcm/v1/vnf_lcm_op_occs/%(id)s'
-                % {"endpoint": CONF.vnf_lcm.endpoint_url,
-                    "id": vnf_lcm_op_occs.id}
+                "href":
+                '{endpoint}/vnflcm/v1/vnf_lcm_op_occs/{id}'.format(
+                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                    id=vnf_lcm_op_occs.id)
             },
             "vnfInstance": {
-                "href": '%(endpoint)s/vnflcm/v1/vnf_instances/%(id)s'
-                % {"endpoint": CONF.vnf_lcm.endpoint_url,
-                    "id": vnf_lcm_op_occs.vnf_instance_id}
+                "href":
+                '{endpoint}/vnflcm/v1/vnf_instances/{id}'.format(
+                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                    id=vnf_lcm_op_occs.vnf_instance_id)
+            },
+            "retry": {
+                "href":
+                '{endpoint}/vnflcm/v1/vnf_lcm_op_occs/{id}/retry'.format(
+                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                    id=vnf_lcm_op_occs.id)
             },
             "rollback": {
                 "href":
-                '%(endpoint)s/vnflcm/v1/vnf_lcm_op_occs/%(id)s/rollback'
-                % {"endpoint": CONF.vnf_lcm.endpoint_url,
-                    "id": vnf_lcm_op_occs.id}
+                '{endpoint}/vnflcm/v1/vnf_lcm_op_occs/{id}/rollback'.format(
+                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                    id=vnf_lcm_op_occs.id)
             },
             "grant": {
-                "href": '%(endpoint)s/vnflcm/v1/vnf_lcm_op_occs/%(id)s/grant'
-                % {"endpoint": CONF.vnf_lcm.endpoint_url,
-                    "id": vnf_lcm_op_occs.id}
+                "href":
+                '{endpoint}/vnflcm/v1/vnf_lcm_op_occs/{id}/grant'.format(
+                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                    id=vnf_lcm_op_occs.id)
+            },
+            "fail": {
+                "href":
+                '{endpoint}/vnflcm/v1/vnf_lcm_op_occs/{id}/fail'.format(
+                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                    id=vnf_lcm_op_occs.id)
             }
         }
 
@@ -129,11 +161,13 @@ class ViewBuilder(base.BaseViewBuilder):
         vnf_instance_dict.update(links)
         return vnf_instance_dict
 
+    # TODO(esto-aln): This method will be transferred to
+    # tacker/api/views/vnf_lcm_op_occs.py in the future
     def _get_vnf_lcm_op_occs(self, vnf_lcm_op_occs):
         vnf_lcm_op_occs_dict = vnf_lcm_op_occs.to_dict()
-        vnf_lcm_op_occs_dict.pop('error_point')
         vnf_lcm_op_occs_dict = utils.convert_snakecase_to_camelcase(
             vnf_lcm_op_occs_dict)
+        vnf_lcm_op_occs_dict.pop('errorPoint')
 
         links = self._get_lcm_op_occs_links(vnf_lcm_op_occs)
 
@@ -158,10 +192,10 @@ class ViewBuilder(base.BaseViewBuilder):
         return {
             "_links": {
                 "self": {
-                    "href": '%(endpoint)s/vnflcm/v1/subscriptions/%(id)s' %
-                    {
-                        "endpoint": CONF.vnf_lcm.endpoint_url,
-                        "id": decode_id}}}}
+                    "href":
+                    '{endpoint}/vnflcm/v1/subscriptions/{id}'.format(
+                        endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
+                        id=decode_id)}}}
 
     def _basic_subscription_info(self, vnf_lcm_subscription, filter=None):
         if not filter:
@@ -183,6 +217,10 @@ class ViewBuilder(base.BaseViewBuilder):
                 'callbackUri': vnf_lcm_subscription.callback_uri,
             }
 
+    # TODO(esto.aln): To remove all processing that are
+    # related to list subscription from vnf_lcm.py and
+    # transfer these to vnf_subscriptions.py, but this
+    # will be handled in a future patch.
     def _subscription_filter(
             self,
             subscription_data,
@@ -246,6 +284,10 @@ class ViewBuilder(base.BaseViewBuilder):
     def subscription_create(self, vnf_lcm_subscription, filter):
         return self._get_vnf_lcm_subscription(vnf_lcm_subscription, filter)
 
+    # TODO(esto.aln): To remove all processing that are
+    # related to list subscription from vnf_lcm.py and
+    # transfer these to vnf_subscriptions.py, but this
+    # will be handled in a future patch.
     def subscription_list(
             self,
             vnf_lcm_subscriptions,
@@ -254,8 +296,13 @@ class ViewBuilder(base.BaseViewBuilder):
         return self._subscription_filter(
             vnf_lcm_subscriptions, nextpage_opaque_marker, paging)
 
+    # TODO(esto.aln): To remove show subscription related processing
+    # in vnf_lcm.py. Current processing for show subscription is in
+    # vnf_subscriptions.py.
     def subscription_show(self, vnf_lcm_subscriptions):
         return self._get_vnf_lcm_subscription(vnf_lcm_subscriptions)
 
+    # TODO(esto-aln): This method will be transferred to
+    # tacker/api/views/vnf_lcm_op_occs.py in the future
     def show_lcm_op_occs(self, vnf_lcm_op_occs):
         return self._get_vnf_lcm_op_occs(vnf_lcm_op_occs)
