@@ -17,6 +17,8 @@
 Tacker base exception handling.
 """
 
+import re
+
 from oslo_log import log as logging
 import webob.exc
 from webob import util as woutil
@@ -206,6 +208,18 @@ class DuplicateResourceName(TackerException):
 class DuplicateEntity(Conflict):
     message = _("%(_type)s already exist with given %(entry)s")
 
+    def __init__(self, *args, **kwargs):
+        # oslo.db does not parse duplicate key error correctly
+        # if MySQL server is >=8.0.19. (oslo.db BUG 1896916)
+        # TODO(kden): revert once the issue is resolved.
+        if len(kwargs["entry"]) == 1:
+            matched = re.match(
+                r"(?P<tbl>[^\.]+)\.uniq_(?P=tbl)0(?P<columns>.+)$",
+                kwargs["entry"][0])
+            if matched is not None:
+                kwargs["entry"] = matched.group("columns").split("0")
+        super(DuplicateEntity, self).__init__(*args, **kwargs)
+
 
 class ValidationError(BadRequest):
     message = "%(detail)s"
@@ -287,10 +301,24 @@ class VnfPreInstantiationFailed(TackerException):
                 "%(error)s")
 
 
+class VnfScaleFailed(TackerException):
+    message = _("Scale Vnf failed for vnf %(id)s, error: %(error)s")
+
+
 class VnfHealFailed(TackerException):
     message = _("Heal Vnf failed for vnf %(id)s, error: %(error)s")
 
 
+class VnfChangeExtConnFailed(TackerException):
+    message = _("Change external connectivity failed "
+                "for vnf %(id)s, error: %(error)s")
+
+
+class VnfChangeExtConnWaitFailed(TackerException):
+    message = _("Change external connectivity wait failed "
+                "for vnf %(id)s, error: %(error)s")
+
+
 class LockCreationFailed(TackerException):
     message = _('Unable to create lock. Coordination backend not started.')
 
@@ -362,3 +390,54 @@ class DBAccessError(TackerException):
 
 class SeeOther(TackerException):
     code = 303
+
+
+class MgmtDriverHashMatchFailure(TackerException):
+    message = _('The hash verification of VNF Package MgmtDriver '
+                'and Tacker MgmtDriver does not match.')
+
+
+class MgmtDriverInconsistent(TackerException):
+    message = _('The %(MgmtDriver)s specified in the VNFD is inconsistent '
+                'with the MgmtDriver in the configuration file.')
+
+
+class MgmtDriverNotFound(TackerException):
+    message = _('The %(param)s in the additionalParams does not exist.')
+
+
+class MgmtDriverParamInvalid(TackerException):
+    message = _('The %(param)s in the additionalParams is invalid.')
+
+
+class MgmtDriverRemoteCommandError(TackerException):
+    message = _('Failed to execute remote command.')
+
+
+class MgmtDriverRemoteCommandTimeOut(TackerException):
+    message = _('The execution of the remote command timed out.')
+
+
+class MgmtDriverOtherError(TackerException):
+    message = _('An error occurred in MgmtDriver: %(error_message)s.')
+
+
+class VnfConflictStateWithErrorPoint(Conflict):
+    message = _("Vnf %(uuid)s in status %(state)s. "
+                "Error point %(error_point)s. "
+                "Cannot %(action)s while the vnf is in this state "
+                "with this error point.")
+
+
+class InvalidIpAddr(TackerException):
+    message = _('Invalid ip address value in resource %(id)s.')
+
+
+class NamespaceIsNotUnique(TackerException):
+    message = _('There are multiple namespaces in the manifest file '
+                'for Kubernetes. Only one namespace can be used in one VNF.')
+
+
+class TenantMatchFailure(TackerException):
+    message = _('The target %(resource)s %(id)s cannot be %(action)s '
+                'from a VIM of a different tenant.')
