@@ -16,7 +16,6 @@
 import abc
 
 from oslo_log import log as logging
-import six
 
 from tacker._i18n import _
 from tacker.api import extensions
@@ -88,6 +87,10 @@ class VNFHealWaitFailed(exceptions.TackerException):
     message = _('VNF Heal %(reason)s')
 
 
+class VNFChangeExtConnWaitFailed(exceptions.TackerException):
+    message = _('VNF ChangeExtConn %(reason)s')
+
+
 class VNFDeleteFailed(exceptions.TackerException):
     message = _('%(reason)s')
 
@@ -100,6 +103,48 @@ class VNFDNotFound(exceptions.NotFound):
     message = _('VNFD %(vnfd_id)s could not be found')
 
 
+class CnfDefinitionNotFound(exceptions.NotFound):
+    message = _(
+        "CNF definition file with path %(path)s "
+        "is not found in vnf_artifacts.")
+
+
+class CNFCreateWaitFailed(exceptions.TackerException):
+    message = _('CNF Create Failed with reason: %(reason)s')
+
+
+class CNFScaleFailed(exceptions.TackerException):
+    message = _('CNF Scale Failed with reason: %(reason)s')
+
+
+class CNFScaleWaitFailed(exceptions.TackerException):
+    message = _('CNF Scale Wait Failed with reason: %(reason)s')
+
+
+class CNFHealFailed(exceptions.TackerException):
+    message = _('%(reason)s')
+
+
+class CNFHealWaitFailed(exceptions.TackerException):
+    message = _('%(reason)s')
+
+
+class InvalidVimConnectionInfo(exceptions.TackerException):
+    message = _('Invalid vim_connection_info: %(reason)s')
+
+
+class HelmClientRemoteCommandError(exceptions.TackerException):
+    message = _('Failed to execute remote command.')
+
+
+class HelmClientMissingParamsError(exceptions.TackerException):
+    message = _('The specified value %(value)s was not found.')
+
+
+class HelmClientOtherError(exceptions.TackerException):
+    message = _('An error occurred in HelmClient: %(error_message)s.')
+
+
 class ServiceTypeNotFound(exceptions.NotFound):
     message = _('service type %(service_type_id)s could not be found')
 
@@ -208,6 +253,15 @@ class InvalidInstReqInfoForScaling(exceptions.InvalidInput):
                 "fixed ip_address or mac_address.")
 
 
+class InvalidMaintenanceParameter(exceptions.InvalidInput):
+    message = _("Could not find the required params for maintenance")
+
+
+class OIDCAuthFailed(exceptions.InvalidInput):
+    message = _("OIDC authentication and authorization failed."
+                " Detail: %(detail)s")
+
+
 def _validate_service_type_list(data, valid_values=None):
     if not isinstance(data, list):
         msg = _("Invalid data format for service list: '%s'") % data
@@ -491,6 +545,37 @@ SUB_RESOURCE_ATTRIBUTE_MAP = {
                 }
             }
         }
+    },
+    'maintenances': {
+        'parent': {
+            'collection_name': 'vnfs',
+            'member_name': 'vnf'
+        },
+        'members': {
+            'maintenance': {
+                'parameters': {
+                    'params': {
+                        'allow_post': True,
+                        'allow_put': False,
+                        'is_visible': True,
+                        'validate': {'type:dict_or_none': None}
+                    },
+                    'tenant_id': {
+                        'allow_post': True,
+                        'allow_put': False,
+                        'validate': {'type:string': None},
+                        'required_by_policy': False,
+                        'is_visible': False
+                    },
+                    'response': {
+                        'allow_post': False,
+                        'allow_put': False,
+                        'validate': {'type:dict_or_none': None},
+                        'is_visible': True
+                    }
+                }
+            }
+        }
     }
 }
 
@@ -562,8 +647,7 @@ class Vnfm(extensions.ExtensionDescriptor):
         return version_map.get(version, {})
 
 
-@six.add_metaclass(abc.ABCMeta)
-class VNFMPluginBase(service_base.NFVPluginBase):
+class VNFMPluginBase(service_base.NFVPluginBase, metaclass=abc.ABCMeta):
     def get_plugin_name(self):
         return constants.VNFM
 
@@ -623,3 +707,7 @@ class VNFMPluginBase(service_base.NFVPluginBase):
     def create_vnf_trigger(
             self, context, vnf_id, trigger):
         pass
+
+    @abc.abstractmethod
+    def create_vnf_maintenance(self, context, vnf_id, maintenance):
+        pass
