@@ -15,7 +15,6 @@
 #    under the License.
 
 import os
-import yaml
 
 from keystoneauth1 import exceptions
 from keystoneauth1 import identity
@@ -28,13 +27,12 @@ from oslo_log import log as logging
 
 from tacker._i18n import _
 from tacker.common import log
+from tacker.common import utils
 from tacker import context as t_context
 from tacker.extensions import nfvo
 from tacker.keymgr import API as KEYMGR_API
-from tacker.mistral import mistral_client
 from tacker.nfvo.drivers.vim import abstract_vim_driver
 from tacker.nfvo.drivers.vnffg import abstract_vnffg_driver
-from tacker.nfvo.drivers.workflow import workflow_generator
 from tacker.nfvo.nfvo_plugin import NfvoPlugin
 from tacker.plugins.common import constants
 from tacker.vnfm import keystone
@@ -60,7 +58,6 @@ OPENSTACK_OPTS = [
                help=_('Number of seconds to wait between packets'))
 ]
 cfg.CONF.register_opts(OPTS, 'vim_keys')
-cfg.CONF.register_opts(OPENSTACK_OPTS, 'vim_monitor')
 
 _VALID_RESOURCE_TYPES = {'network': {'client': neutron_client.Client,
                                      'cmd': 'list_networks',
@@ -86,7 +83,7 @@ SFC_ENCAP = 'sfc_encap'
 
 
 def config_opts():
-    return [('vim_keys', OPTS), ('vim_monitor', OPENSTACK_OPTS)]
+    return [('vim_keys', OPTS)]
 
 
 class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
@@ -117,16 +114,16 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
 
         Initialize keystoneclient with provided authentication attributes.
         """
-        verify = 'True' == vim_obj['auth_cred'].get('cert_verify', 'True') \
-                 or False
+        verify = utils.str_to_bool(vim_obj['auth_cred']
+                                   .get('cert_verify', 'True'))
         auth_url = vim_obj['auth_url']
-        keystone_version = NfvoPlugin.validate_keystone_auth_url(
+        NfvoPlugin.validate_keystone_auth_url(
             auth_url=auth_url,
             verify=verify)
-        auth_cred = self._get_auth_creds(vim_obj, keystone_version)
+        auth_cred = self._get_auth_creds(vim_obj)
         return self._initialize_keystone(auth_cred)
 
-    def _get_auth_creds(self, vim_obj, keystone_version):
+    def _get_auth_creds(self, vim_obj):
         auth_cred = vim_obj['auth_cred']
         vim_project = vim_obj['vim_project']
         auth_cred['project_id'] = vim_project.get('id')
@@ -134,9 +131,8 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
         auth_cred['project_domain_name'] = vim_project.get(
             'project_domain_name')
         auth_cred['auth_url'] = vim_obj['auth_url']
-        if keystone_version not in auth_cred['auth_url']:
-            auth_cred['auth_url'] = auth_cred['auth_url'] + '/' + \
-                keystone_version
+        if 'v3' not in auth_cred['auth_url']:
+            auth_cred['auth_url'] = f'{auth_cred["auth_url"]}/v3'
         return auth_cred
 
     def _get_auth_plugin(self, **kwargs):
@@ -145,13 +141,12 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
         return auth_plugin
 
     def _initialize_keystone(self, auth):
-        ks_client = self.keystone.initialize_client(**auth)
-        return ks_client
+        return self.keystone.initialize_client(**auth)
 
     def _find_regions(self, ks_client):
-        region_info = ks_client.regions.list()
-        region_list = [region.id for region in region_info]
-        return region_list
+        # TODO(h-asahina): implement this method into KeystoneClient module
+        resp = ks_client.get('/v3/regions')
+        return [region['id'] for region in resp.json().get('regions', [])]
 
     def discover_placement_attr(self, vim_obj, ks_client):
         """Fetch VIM placement information
@@ -159,11 +154,11 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
         Attributes can include regions, AZ.
         """
         try:
-            regions_list = self._find_regions(ks_client)
+            regions = self._find_regions(ks_client)
         except (exceptions.Unauthorized, exceptions.BadRequest) as e:
-            LOG.warning("Authorization failed for user")
+            LOG.error("Authorization failed for user")
             raise nfvo.VimUnauthorizedException(message=e.message)
-        vim_obj['placement_attr'] = {'regions': regions_list}
+        vim_obj['placement_attr'] = {'regions': regions}
         return vim_obj
 
     @log.log
@@ -206,9 +201,8 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
                 LOG.debug('VIM key deleted successfully for vim %s',
                           vim_id)
             except Exception as ex:
-                LOG.warning('VIM key deletion failed for vim %s due to %s',
-                            vim_id,
-                            ex)
+                LOG.error('VIM key deletion failed for vim %s due to %s',
+                          vim_id, ex)
                 raise
         else:
             key_file = os.path.join(CONF.vim_keys.openstack, vim_id)
@@ -242,9 +236,8 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
                 LOG.debug('VIM auth successfully stored for vim %s',
                           vim_id)
             except Exception as ex:
-                LOG.warning('VIM key creation failed for vim %s due to %s',
-                            vim_id,
-                            ex)
+                LOG.error('VIM key creation failed for vim %s due to %s',
+                          vim_id, ex)
                 raise
 
         else:
@@ -305,12 +298,12 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
         :param client_type: openstack client to initialize
         :return: initialized client
         """
-        verify = 'True' == vim_obj.get('cert_verify', 'True') or False
+        verify = utils.str_to_bool(vim_obj.get('cert_verify', 'True'))
         auth_url = vim_obj['auth_url']
-        keystone_version = NfvoPlugin.validate_keystone_auth_url(
+        NfvoPlugin.validate_keystone_auth_url(
             auth_url=auth_url,
             verify=verify)
-        auth_cred = self._get_auth_creds(vim_obj, keystone_version)
+        auth_cred = self._get_auth_creds(vim_obj)
         auth_plugin = self._get_auth_plugin(**auth_cred)
         sess = session.Session(auth=auth_plugin)
         return client_type(session=sess)
@@ -711,46 +704,6 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
         neutronclient_ = NeutronClient(auth_attr)
         neutronclient_.flow_classifier_delete(fc_id)
 
-    def get_mistral_client(self, auth_dict):
-        if not auth_dict:
-            LOG.warning("auth dict required to instantiate mistral client")
-            raise EnvironmentError('auth dict required for'
-                                   ' mistral workflow driver')
-        return mistral_client.MistralClient(
-            keystone.Keystone().initialize_client(**auth_dict),
-            auth_dict['token']).get_client()
-
-    def prepare_and_create_workflow(self, resource, action,
-                                    kwargs, auth_dict=None):
-        mistral_client = self.get_mistral_client(auth_dict)
-        wg = workflow_generator.WorkflowGenerator(resource, action)
-        wg.task(**kwargs)
-        if not wg.get_tasks():
-            raise nfvo.NoTasksException(resource=resource, action=action)
-        yaml.SafeDumper.ignore_aliases = lambda self, data: True
-        definition_yaml = yaml.safe_dump(wg.definition)
-        workflow = mistral_client.workflows.create(definition_yaml)
-        return {'id': workflow[0].id, 'input': wg.get_input_dict()}
-
-    def execute_workflow(self, workflow, auth_dict=None):
-        return self.get_mistral_client(auth_dict) \
-            .executions.create(
-            workflow_identifier=workflow['id'],
-            workflow_input=workflow['input'],
-            wf_params={})
-
-    def get_execution(self, execution_id, auth_dict=None):
-        return self.get_mistral_client(auth_dict) \
-            .executions.get(execution_id)
-
-    def delete_execution(self, execution_id, auth_dict=None):
-        return self.get_mistral_client(auth_dict).executions \
-            .delete(execution_id, force=True)
-
-    def delete_workflow(self, workflow_id, auth_dict=None):
-        return self.get_mistral_client(auth_dict) \
-            .workflows.delete(workflow_id)
-
     def _delete_ppgs_and_pps(self, neutronclient, new_ppgs,
                              past_ppgs_dict, pcs_list, fc_ids):
         if new_ppgs:
@@ -806,7 +759,7 @@ class NeutronClient(object):
 
     def __init__(self, auth_attr):
         auth_cred = auth_attr.copy()
-        verify = 'True' == auth_cred.pop('cert_verify', 'True') or False
+        verify = utils.str_to_bool(auth_cred.pop('cert_verify', 'True'))
         auth = identity.Password(**auth_cred)
         sess = session.Session(auth=auth, verify=verify)
         self.client = neutron_client.Client(session=sess)
@@ -839,7 +792,7 @@ class NeutronClient(object):
         try:
             self.client.delete_sfc_flow_classifier(fc_id)
         except nc_exceptions.NotFound:
-            LOG.warning("fc %s not found", fc_id)
+            LOG.error("fc %s not found", fc_id)
             raise ValueError('fc %s not found' % fc_id)
 
     def port_pair_create(self, port_pair_dict):
@@ -863,7 +816,7 @@ class NeutronClient(object):
         try:
             self.client.delete_sfc_port_pair(port_pair_id)
         except nc_exceptions.NotFound:
-            LOG.warning('port pair %s not found', port_pair_id)
+            LOG.error('port pair %s not found', port_pair_id)
             raise ValueError('port pair %s not found' % port_pair_id)
 
     def port_pair_group_create(self, ppg_dict):
@@ -871,7 +824,7 @@ class NeutronClient(object):
             ppg = self.client.create_sfc_port_pair_group(
                 {'port_pair_group': ppg_dict})
         except nc_exceptions.BadRequest as e:
-            LOG.warning('create port pair group returns %s', e)
+            LOG.error('create port pair group returns %s', e)
             raise ValueError(str(e))
 
         if ppg and len(ppg):
@@ -887,7 +840,7 @@ class NeutronClient(object):
         try:
             self.client.delete_sfc_port_pair_group(ppg_id)
         except nc_exceptions.NotFound:
-            LOG.warning('port pair group %s not found', ppg_id)
+            LOG.error('port pair group %s not found', ppg_id)
             raise ValueError('port pair group %s not found' % ppg_id)
 
     def port_chain_create(self, port_chain_dict):
@@ -895,7 +848,7 @@ class NeutronClient(object):
             pc = self.client.create_sfc_port_chain(
                 {'port_chain': port_chain_dict})
         except nc_exceptions.BadRequest as e:
-            LOG.warning('create port chain returns %s', e)
+            LOG.error('create port chain returns %s', e)
             raise ValueError(str(e))
 
         if pc and len(pc):
@@ -934,7 +887,7 @@ class NeutronClient(object):
                                         pp_id = port_pairs[j]
                                         self.client.delete_sfc_port_pair(pp_id)
         except nc_exceptions.NotFound:
-            LOG.warning('port chain %s not found', port_chain_id)
+            LOG.error('port chain %s not found', port_chain_id)
             raise ValueError('port chain %s not found' % port_chain_id)
 
     def port_chain_update(self, port_chain_id, port_chain):
@@ -942,7 +895,7 @@ class NeutronClient(object):
             pc = self.client.update_sfc_port_chain(port_chain_id,
                                     {'port_chain': port_chain})
         except nc_exceptions.BadRequest as e:
-            LOG.warning('update port chain returns %s', e)
+            LOG.error('update port chain returns %s', e)
             raise ValueError(str(e))
         if pc and len(pc):
             return pc['port_chain']['id']
@@ -973,5 +926,5 @@ class NeutronClient(object):
 
             return port_pair_group
         except nc_exceptions.NotFound:
-            LOG.warning('port pair group %s not found', ppg_id)
+            LOG.error('port pair group %s not found', ppg_id)
             raise ValueError('port pair group %s not found' % ppg_id)
