@@ -20,7 +20,6 @@ from oslo_utils import uuidutils
 from oslo_versionedobjects import base as ovoo_base
 from sqlalchemy import exc
 from sqlalchemy.orm import joinedload
-from sqlalchemy_filters import apply_filters
 
 from tacker._i18n import _
 from tacker.common import exceptions
@@ -28,23 +27,27 @@ from tacker.common import utils
 from tacker.db import api as db_api
 from tacker.db.db_sqlalchemy import api
 from tacker.db.db_sqlalchemy import models
+from tacker.db import sqlalchemyutils
 from tacker.db.vnfm import vnfm_db
 from tacker import objects
 from tacker.objects import base
+from tacker.objects import common
 from tacker.objects import fields
 from tacker.objects import vnf_instantiated_info
 from tacker.objects import vnf_package as vnf_package_obj
-from tacker.objects import vnf_package_vnfd as vnf_package_vnfd
+from tacker.objects import vnf_package_vnfd
 
 
 LOG = logging.getLogger(__name__)
 
 
 @db_api.context_manager.reader
-def _vnf_instance_get_by_id(context, vnf_instance_id, columns_to_join=None):
+def _vnf_instance_get_by_id(context, vnf_instance_id, columns_to_join=None,
+                            read_deleted="no"):
 
     query = api.model_query(context, models.VnfInstance,
-                            read_deleted="no", project_only=True). \
+                            read_deleted=read_deleted,
+                            project_only=True). \
         filter_by(id=vnf_instance_id)
 
     if columns_to_join:
@@ -120,11 +123,28 @@ def _vnf_instance_list_by_filter(context, columns_to_join=None,
             query = query.options(joinedload(column))
 
     if filters:
-        query = apply_filters(query, filters)
+        query = common.apply_filters(query, filters)
 
     return query.all()
 
 
+@db_api.context_manager.reader
+def _vnf_instance_list_by_filter_query(context, columns_to_join=None,
+                                 filters=None):
+    query = api.model_query(context, models.VnfInstance,
+                            read_deleted="no",
+                            project_only=True)
+
+    if columns_to_join:
+        for column in columns_to_join:
+            query = query.options(joinedload(column))
+
+    if filters:
+        query = common.apply_filters(query, filters)
+
+    return query
+
+
 def _make_vnf_instance_list(context, vnf_instance_list, db_vnf_instance_list,
                             expected_attrs):
     vnf_instance_cls = VnfInstance
@@ -334,20 +354,20 @@ class VnfInstance(base.TackerObject, base.TackerPersistentObject,
     }
 
     ALL_ATTRIBUTES = {
-        'id': ('id', "string", 'VnfInstance'),
+        'id': ('id', "uuid", 'VnfInstance'),
         'vnfInstanceName': ('vnf_instance_name', 'string', 'VnfInstance'),
         'vnfInstanceDescription': (
             'vnf_instance_description', 'string', 'VnfInstance'),
         'instantiationState': ('instantiation_state', 'string', 'VnfInstance'),
         'taskState': ('task_state', 'string', 'VnfInstance'),
-        'vnfdId': ('vnfd_id', 'string', 'VnfInstance'),
+        'vnfdId': ('vnfd_id', 'uuid', 'VnfInstance'),
         'vnfProvider': ('vnf_provider', 'string', 'VnfInstance'),
         'vnfProductName': ('vnf_product_name', 'string', 'VnfInstance'),
         'vnfSoftwareVersion': (
             'vnf_software_version', 'string', 'VnfInstance'),
         'vnfdVersion': ('vnfd_version', 'string', 'VnfInstance'),
         'tenantId': ('tenant_id', 'string', 'VnfInstance'),
-        'vnfPkgId': ('vnf_pkg_id', 'string', 'VnfInstance'),
+        'vnfPkgId': ('vnf_pkg_id', 'uuid', 'VnfInstance'),
         'vimConnectionInfo/*': ('vim_connection_info', 'key_value_pair',
                                 {"key_column": "key", "value_column": "value",
                                  "model": "VnfInstance"}),
@@ -517,10 +537,11 @@ class VnfInstance(base.TackerObject, base.TackerPersistentObject,
             vnfd_id)
 
     @base.remotable_classmethod
-    def get_by_id(cls, context, id):
+    def get_by_id(cls, context, id, read_deleted="no"):
         expected_attrs = ["instantiated_vnf_info"]
         db_vnf_instance = _vnf_instance_get_by_id(
-            context, id, columns_to_join=expected_attrs)
+            context, id, columns_to_join=expected_attrs,
+            read_deleted=read_deleted)
         return cls._from_db_object(context, cls(), db_vnf_instance,
                                    expected_attrs=expected_attrs)
 
@@ -542,6 +563,22 @@ class VnfInstanceList(ovoo_base.ObjectListBase, base.TackerObject):
         return _make_vnf_instance_list(context, cls(), db_vnf_instances,
                                        expected_attrs)
 
+    @base.remotable_classmethod
+    def get_by_marker_filter(cls, context, limit, marker_obj,
+            filters=None, expected_attrs=None):
+        expected_attrs = ["instantiated_vnf_info"]
+        query = _vnf_instance_list_by_filter_query(context,
+                columns_to_join=expected_attrs,
+                filters=filters)
+        query = sqlalchemyutils.paginate_query(query, model=models.VnfInstance,
+            limit=limit,
+            sorts=[['id', 'asc']],
+            marker_obj=marker_obj)
+        db_vnf_instances = query.all()
+
+        return _make_vnf_instance_list(context, cls(), db_vnf_instances,
+                expected_attrs)
+
     @base.remotable_classmethod
     def get_by_filters(cls, context, filters=None,
                        expected_attrs=None):
