@@ -13,8 +13,31 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+
+import threading
+
+from oslo_log import log as logging
+
+
+from tacker.sol_refactored.api import api_version
+from tacker.sol_refactored.common import config
 from tacker.sol_refactored.common import exceptions as sol_ex
+from tacker.sol_refactored.common import http_client
+from tacker.sol_refactored.common import subscription_utils
 from tacker.sol_refactored.common import vnfd_utils
+from tacker.sol_refactored import objects
+
+
+LOG = logging.getLogger(__name__)
+
+CONF = config.CONF
+
+TEST_NOTIFICATION_TIMEOUT = 20  # seconds
+NOTIFY_TYPE_PM = 'PM'
+NOTIFY_TYPE_FM = 'FM'
+AUTH_TYPE_OAUTH2_CLIENT_CERT = 'OAUTH2_CLIENT_CERT'
+AUTH_TYPE_OAUTH2_CLIENT_CREDENTIALS = 'OAUTH2_CLIENT_CREDENTIALS'
+AUTH_TYPE_BASIC = 'BASIC'
 
 
 def get_vnfd(vnfd_id, csar_dir):
@@ -23,47 +46,6 @@ def get_vnfd(vnfd_id, csar_dir):
     return vnfd
 
 
-def get_vdu_info(grant, inst, vnfd):
-    volume_name = ''
-    volume_size = ''
-    flavour_id = inst['instantiatedVnfInfo']['flavourId']
-    vdu_nodes = vnfd.get_vdu_nodes(flavour_id)
-    storage_nodes = vnfd.get_storage_nodes(flavour_id)
-    vdu_info_dict = {}
-    for name, node in vdu_nodes.items():
-        flavor = get_param_flavor(name, flavour_id, vnfd, grant)
-        image = get_param_image(name, flavour_id, vnfd, grant)
-        vdu_storage_names = vnfd.get_vdu_storages(node)
-        for vdu_storage_name in vdu_storage_names:
-            if storage_nodes[vdu_storage_name].get(
-                    'properties', {}).get('sw_image_data'):
-                image = get_param_image(vdu_storage_name, flavour_id, vnfd,
-                                        grant)
-                volume_name = vdu_storage_name
-                volume_size = storage_nodes[vdu_storage_name].get(
-                    'properties', {}).get(
-                    'virtual_block_storage_data', '').get(
-                    'size_of_storage', ''
-                )
-                volume_size = volume_size.rstrip(' GB')
-                if not volume_size.isdigit():
-                    raise sol_ex.VmRunningFailed(
-                        error_info='The volume size set in VNFD is invalid.')
-                break
-
-        vdu_info_dict[name] = {
-            "flavor": flavor,
-            "image": image
-        }
-
-        if volume_name:
-            vdu_info_dict[name]['volume_info'] = {
-                "volume_name": volume_name,
-                "volume_size": volume_size
-            }
-    return vdu_info_dict
-
-
 def init_nfv_dict(hot_template):
     get_params = []
 
@@ -117,7 +99,7 @@ def get_param_flavor(vdu_name, flavour_id, vnfd, grant):
     return vnfd.get_compute_flavor(flavour_id, vdu_name)
 
 
-def get_param_image(vdu_name, flavour_id, vnfd, grant):
+def get_param_image(vdu_name, flavour_id, vnfd, grant, fallback_vnfd=True):
     # try to get from grant
     if 'vimAssets' in grant:
         assets = grant['vimAssets']
@@ -127,13 +109,18 @@ def get_param_image(vdu_name, flavour_id, vnfd, grant):
                 if image['vnfdSoftwareImageId'] == vdu_name:
                     return image['vimSoftwareImageId']
 
-    # if specified in VNFD, use it
+    if fallback_vnfd:
+        # if this flag is True, VNFD is refered to.
+        # if specified in VNFD, use it.
+        # NOTE: image name is assumed to be unique in the system.
+        # NFVO should be return vimAssets basically.
+        sw_images = vnfd.get_sw_image(flavour_id)
+        for name, image in sw_images.items():
+            if name == vdu_name:
+                return image
+
     # NOTE: if not found. parameter is set to None.
     #       may be error when stack create
-    sw_images = vnfd.get_sw_image(flavour_id)
-    for name, image in sw_images.items():
-        if name == vdu_name:
-            return image
 
 
 def get_param_zone(vdu_name, grant_req, grant):
@@ -151,6 +138,20 @@ def get_param_zone(vdu_name, grant_req, grant):
                             return zone['zoneId']
 
 
+def get_param_zone_by_vnfc(res_id, grant):
+    if 'zones' not in grant or 'addResources' not in grant:
+        return
+
+    for res in grant['addResources']:
+        if res['resourceDefinitionId'] == res_id:
+            if 'zoneId' not in res:
+                return
+            for zone in grant['zones']:
+                if zone['id'] == res['zoneId']:  # must be found
+                    return zone['zoneId']
+            return
+
+
 def get_current_capacity(vdu_name, inst):
     count = 0
     inst_vnfcs = (inst.get('instantiatedVnfInfo', {})
@@ -236,14 +237,9 @@ def get_param_fixed_ips_from_inst(cp_name, inst):
                 return _get_fixed_ips_from_extcp(extcp)
 
 
-def apply_ext_managed_vls(hot_dict, req, grant):
-    # see grant first then instantiateVnfRequest
-    mgd_vls = (grant.get('extManagedVirtualLinks', []) +
-               req.get('extManagedVirtualLinks', []))
-
+def _apply_ext_managed_vls(hot_dict, mgd_vls):
     # NOTE: refer HOT only here, not refer VNFD.
     # HOT and VNFD must be consistent.
-
     for mgd_vl in mgd_vls:
         vl_name = mgd_vl['vnfVirtualLinkDescId']
         network_id = mgd_vl['resourceId']
@@ -277,3 +273,131 @@ def apply_ext_managed_vls(hot_dict, req, grant):
 
         for res_name in del_reses:
             hot_dict['resources'].pop(res_name)
+
+
+def apply_ext_managed_vls(hot_dict, req, grant):
+    # see grant first then instantiateVnfRequest
+    mgd_vls = (grant.get('extManagedVirtualLinks', []) +
+               req.get('extManagedVirtualLinks', []))
+
+    _apply_ext_managed_vls(hot_dict, mgd_vls)
+
+
+def apply_ext_managed_vls_from_inst(hot_dict, inst):
+    mgd_vls = inst['instantiatedVnfInfo'].get('extManagedVirtualLinkInfo')
+
+    if mgd_vls:
+        # convert ExtVirtualLinkInfo to ExtManagedVirtualLinkData.
+        # necessary members only.
+        mgd_vls = [
+            {
+                'vnfVirtualLinkDescId': mgd_vl['vnfVirtualLinkDescId'],
+                'resourceId': mgd_vl['networkResource']['resourceId']
+            }
+            for mgd_vl in mgd_vls
+        ]
+        _apply_ext_managed_vls(hot_dict, mgd_vls)
+
+
+def get_notification_auth_handle(obj_data):
+    auth_req = obj_data.get('authentication', None)
+    if auth_req:
+        auth = objects.SubscriptionAuthentication(
+            authType=auth_req['authType']
+        )
+        if AUTH_TYPE_OAUTH2_CLIENT_CERT in auth.authType:
+            param = obj_data.authentication.paramsOauth2ClientCert
+            ca_cert = CONF.v2_vnfm.notification_mtls_ca_cert_file
+            client_cert = CONF.v2_vnfm.notification_mtls_client_cert_file
+            return http_client.OAuth2MtlsAuthHandle(
+                None, param.tokenEndpoint,
+                param.clientId, ca_cert, client_cert)
+        elif AUTH_TYPE_OAUTH2_CLIENT_CREDENTIALS in auth.authType:
+            param = obj_data.authentication.paramsOauth2ClientCredentials
+            verify = CONF.v2_vnfm.notification_verify_cert
+            if verify and CONF.v2_vnfm.notification_ca_cert_file:
+                verify = CONF.v2_vnfm.notification_ca_cert_file
+            return http_client.OAuth2AuthHandle(
+                None, param.tokenEndpoint, param.clientId,
+                param.clientPassword,
+                verify=verify)
+        elif AUTH_TYPE_BASIC in auth.authType:
+            param = obj_data.authentication.paramsBasic
+            verify = CONF.v2_vnfm.notification_verify_cert
+            if verify and CONF.v2_vnfm.notification_ca_cert_file:
+                verify = CONF.v2_vnfm.notification_ca_cert_file
+            return http_client.BasicAuthHandle(
+                param.userName, param.password,
+                verify=verify)
+        else:
+            raise sol_ex.AuthTypeNotFound(auth.authType)
+    else:
+        verify = CONF.v2_vnfm.notification_verify_cert
+        if verify and CONF.v2_vnfm.notification_ca_cert_file:
+            verify = CONF.v2_vnfm.notification_ca_cert_file
+        return http_client.NoAuthHandle(verify=verify)
+
+
+def async_call(func):
+    def inner(*args, **kwargs):
+        th = threading.Thread(target=func, args=args,
+                              kwargs=kwargs, daemon=True)
+        th.start()
+    return inner
+
+
+@async_call
+def send_notification(obj_data, notif_data, notify_type=None):
+    version = api_version.CURRENT_VERSION
+    auth_handle = subscription_utils.get_notification_auth_handle(obj_data)
+    connect_retries = (CONF.v2_vnfm.notify_connect_retries
+                       if CONF.v2_vnfm.notify_connect_retries else None)
+    client = http_client.HttpClient(auth_handle,
+                                    version=version,
+                                    connect_retries=connect_retries)
+    if notify_type == NOTIFY_TYPE_PM:
+        version = api_version.CURRENT_PM_VERSION
+        auth_handle = get_notification_auth_handle(obj_data)
+        client = http_client.HttpClient(auth_handle,
+                                        version=version)
+    if notify_type == NOTIFY_TYPE_FM:
+        version = api_version.CURRENT_FM_VERSION
+        auth_handle = get_notification_auth_handle(obj_data)
+        client = http_client.HttpClient(auth_handle,
+                                        version=version)
+
+    url = obj_data.callbackUri
+    try:
+        resp, _ = client.do_request(
+            url, "POST", expected_status=[204], body=notif_data)
+    except sol_ex.SolException as ex:
+        # it may occur if test_notification was not executed.
+        LOG.exception(f"send_notification failed: {ex}")
+
+    if resp.status_code != 204:
+        LOG.error(f"send_notification failed: {resp.__dict__}")
+
+
+def test_notification(obj_data, notify_type=None):
+    version = api_version.CURRENT_VERSION
+    auth_handle = subscription_utils.get_notification_auth_handle(obj_data)
+    if notify_type == NOTIFY_TYPE_PM:
+        version = api_version.CURRENT_PM_VERSION
+        auth_handle = get_notification_auth_handle(obj_data)
+    if notify_type == NOTIFY_TYPE_FM:
+        version = api_version.CURRENT_FM_VERSION
+        auth_handle = get_notification_auth_handle(obj_data)
+
+    client = http_client.HttpClient(auth_handle,
+                                    version=version,
+                                    timeout=TEST_NOTIFICATION_TIMEOUT)
+
+    url = obj_data.callbackUri
+    try:
+        resp, _ = client.do_request(url, "GET", expected_status=[204])
+    except sol_ex.SolException as e:
+        # any notify_type of error is considered. avoid 500 error.
+        raise sol_ex.TestNotificationFailed() from e
+
+    if resp.status_code != 204:
+        raise sol_ex.TestNotificationFailed()
