@@ -14,15 +14,15 @@
 #    under the License.
 
 
-from datetime import datetime
-
 from oslo_log import log as logging
 from oslo_utils import uuidutils
 
-from tacker.sol_refactored.api.api_version import supported_versions_v2
+from tacker.sol_refactored.api import api_version
+from tacker.sol_refactored.api.policies.vnflcm_v2 import POLICY_NAME
 from tacker.sol_refactored.api.schemas import vnflcm_v2 as schema
 from tacker.sol_refactored.api import validator
 from tacker.sol_refactored.api import wsgi as sol_wsgi
+from tacker.sol_refactored.common import common_script_utils
 from tacker.sol_refactored.common import config
 from tacker.sol_refactored.common import coordinate
 from tacker.sol_refactored.common import exceptions as sol_ex
@@ -30,6 +30,7 @@ from tacker.sol_refactored.common import lcm_op_occ_utils as lcmocc_utils
 from tacker.sol_refactored.common import subscription_utils as subsc_utils
 from tacker.sol_refactored.common import vim_utils
 from tacker.sol_refactored.common import vnf_instance_utils as inst_utils
+from tacker.sol_refactored.common import vnflcm_utils
 from tacker.sol_refactored.conductor import conductor_rpc_v2
 from tacker.sol_refactored.controller import vnflcm_view
 from tacker.sol_refactored.nfvo import nfvo_client
@@ -53,7 +54,7 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
         self._subsc_view = vnflcm_view.SubscriptionViewBuilder(self.endpoint)
 
     def api_versions(self, request):
-        return sol_wsgi.SolResponse(200, supported_versions_v2)
+        return sol_wsgi.SolResponse(200, api_version.supported_versions_v2)
 
     @validator.schema(schema.CreateVnfRequest_V200, '2.0.0')
     def create(self, request, body):
@@ -62,6 +63,10 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
 
         pkg_info = self.nfvo_client.get_vnf_package_info_vnfd(
             context, vnfd_id)
+        if config.CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(POLICY_NAME.format('create'),
+                        target={'vendor': pkg_info.vnfProvider})
+
         if pkg_info.operationalState != "ENABLED":
             raise sol_ex.VnfdIdNotEnabled(vnfd_id=vnfd_id)
 
@@ -118,15 +123,26 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
 
         selector = self._inst_view.parse_selector(request.GET)
 
-        insts = inst_utils.get_inst_all(request.context)
+        pager = self._inst_view.parse_pager(request)
 
-        resp_body = self._inst_view.detail_list(insts, filters, selector)
+        insts = inst_utils.get_inst_all(request.context,
+                                        marker=pager.marker)
+        if config.CONF.oslo_policy.enhanced_tacker_policy:
+            insts = [inst for inst in insts if request.context.can(
+                POLICY_NAME.format('index'),
+                target=self._get_policy_target(inst),
+                fatal=False)]
 
-        return sol_wsgi.SolResponse(200, resp_body)
+        resp_body = self._inst_view.detail_list(insts, filters,
+                                                selector, pager)
+
+        return sol_wsgi.SolResponse(200, resp_body, link=pager.get_link())
 
     def show(self, request, id):
         inst = inst_utils.get_inst(request.context, id)
-
+        if config.CONF.oslo_policy.enhanced_tacker_policy:
+            request.context.can(POLICY_NAME.format('show'),
+                                target=self._get_policy_target(inst))
         resp_body = self._inst_view.detail(inst)
 
         return sol_wsgi.SolResponse(200, resp_body)
@@ -135,7 +151,9 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
     def delete(self, request, id):
         context = request.context
         inst = inst_utils.get_inst(context, id)
-
+        if config.CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(POLICY_NAME.format('delete'),
+                        target=self._get_policy_target(inst))
         if inst.instantiationState != 'NOT_INSTANTIATED':
             raise sol_ex.VnfInstanceIsInstantiated(inst_id=id)
 
@@ -149,28 +167,14 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
                                                        self.endpoint)
         return sol_wsgi.SolResponse(204, None)
 
-    def _new_lcmocc(self, inst_id, operation, req_body,
-            op_state=v2fields.LcmOperationStateType.STARTING):
-        now = datetime.utcnow()
-        lcmocc = objects.VnfLcmOpOccV2(
-            id=uuidutils.generate_uuid(),
-            operationState=op_state,
-            stateEnteredTime=now,
-            startTime=now,
-            vnfInstanceId=inst_id,
-            operation=operation,
-            isAutomaticInvocation=False,
-            isCancelPending=False,
-            operationParams=req_body)
-
-        return lcmocc
-
     @validator.schema(schema.VnfInfoModificationRequest_V200, '2.0.0')
     @coordinate.lock_vnf_instance('{id}')
     def update(self, request, id, body):
         context = request.context
         inst = inst_utils.get_inst(context, id)
-
+        if config.CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(POLICY_NAME.format('update'),
+                        target=self._get_policy_target(inst))
         lcmocc_utils.check_lcmocc_in_progress(context, id)
         if (inst.instantiationState == 'NOT_INSTANTIATED'
                 and 'vimConnectionInfo' in body):
@@ -201,7 +205,8 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
                         detail="vnfcInstanceId(%s) does not exist."
                         % vnfc_mod['id'])
 
-        lcmocc = self._new_lcmocc(id, v2fields.LcmOperationType.MODIFY_INFO,
+        lcmocc = vnflcm_utils.new_lcmocc(
+            id, v2fields.LcmOperationType.MODIFY_INFO,
             body, v2fields.LcmOperationStateType.PROCESSING)
         lcmocc.create(context)
 
@@ -211,19 +216,62 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
 
         return sol_wsgi.SolResponse(202, None, location=location)
 
+    def _get_policy_target(self, vnf_instance):
+        vendor = vnf_instance.vnfProvider
+
+        if vnf_instance.instantiationState == 'NOT_INSTANTIATED':
+            area = '*'
+            tenant = '*'
+        else:
+            vim_type = None
+            area = None
+            if vnf_instance.obj_attr_is_set('vimConnectionInfo'):
+                for _, vim_conn_info in vnf_instance.vimConnectionInfo.items():
+                    area = vim_conn_info.get('extra', {}).get('area')
+                    vim_type = vim_conn_info.vimType
+                    if area and vim_type:
+                        break
+
+            tenant = None
+            if (vnf_instance.obj_attr_is_set('instantiatedVnfInfo') and
+                    vnf_instance.instantiatedVnfInfo.obj_attr_is_set(
+                        'metadata')):
+
+                tenant = (vnf_instance.instantiatedVnfInfo
+                          .metadata.get('namespace'))
+
+            # TODO(kexuesheng): Add steps to get tenant of VNFs deployed
+            #  in OpenStack VIM. This is a temporary workaround until that
+            #  information is available.
+            if vim_type == "ETSINFV.OPENSTACK_KEYSTONE.V_3":
+                tenant = '*'
+
+        target = {
+            'vendor': vendor,
+            'area': area,
+            'tenant': tenant
+        }
+
+        target = {k: v for k, v in target.items() if v is not None}
+
+        return target
+
     @validator.schema(schema.InstantiateVnfRequest_V200, '2.0.0')
     @coordinate.lock_vnf_instance('{id}')
     def instantiate(self, request, id, body):
         context = request.context
         inst = inst_utils.get_inst(context, id)
+        if config.CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(POLICY_NAME.format('instantiate'),
+                        target=self._get_policy_target(inst))
 
         if inst.instantiationState != 'NOT_INSTANTIATED':
             raise sol_ex.VnfInstanceIsInstantiated(inst_id=id)
 
         lcmocc_utils.check_lcmocc_in_progress(context, id)
 
-        lcmocc = self._new_lcmocc(id, v2fields.LcmOperationType.INSTANTIATE,
-                                  body)
+        lcmocc = vnflcm_utils.new_lcmocc(
+            id, v2fields.LcmOperationType.INSTANTIATE, body)
 
         req_param = lcmocc.operationParams
         # if there is partial vimConnectionInfo check and fulfill here.
@@ -250,14 +298,17 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
     def terminate(self, request, id, body):
         context = request.context
         inst = inst_utils.get_inst(context, id)
+        if config.CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(POLICY_NAME.format('terminate'),
+                        target=self._get_policy_target(inst))
 
         if inst.instantiationState != 'INSTANTIATED':
             raise sol_ex.VnfInstanceIsNotInstantiated(inst_id=id)
 
         lcmocc_utils.check_lcmocc_in_progress(context, id)
 
-        lcmocc = self._new_lcmocc(id, v2fields.LcmOperationType.TERMINATE,
-                                  body)
+        lcmocc = vnflcm_utils.new_lcmocc(
+            id, v2fields.LcmOperationType.TERMINATE, body)
         lcmocc.create(context)
 
         self.conductor_rpc.start_lcm_op(context, lcmocc.id)
@@ -266,93 +317,29 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
 
         return sol_wsgi.SolResponse(202, None, location=location)
 
-    def _get_current_scale_level(self, inst, aspect_id):
-        if (inst.obj_attr_is_set('instantiatedVnfInfo') and
-                inst.instantiatedVnfInfo.obj_attr_is_set('scaleStatus')):
-            for scale_info in inst.instantiatedVnfInfo.scaleStatus:
-                if scale_info.aspectId == aspect_id:
-                    return scale_info.scaleLevel
-
-    def _get_max_scale_level(self, inst, aspect_id):
-        if (inst.obj_attr_is_set('instantiatedVnfInfo') and
-                inst.instantiatedVnfInfo.obj_attr_is_set('maxScaleLevels')):
-            for scale_info in inst.instantiatedVnfInfo.maxScaleLevels:
-                if scale_info.aspectId == aspect_id:
-                    return scale_info.scaleLevel
-
     @validator.schema(schema.ScaleVnfRequest_V200, '2.0.0')
-    @coordinate.lock_vnf_instance('{id}')
     def scale(self, request, id, body):
         context = request.context
         inst = inst_utils.get_inst(context, id)
+        if config.CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(POLICY_NAME.format('scale'),
+                        target=self._get_policy_target(inst))
 
-        if inst.instantiationState != 'INSTANTIATED':
-            raise sol_ex.VnfInstanceIsNotInstantiated(inst_id=id)
-
-        lcmocc_utils.check_lcmocc_in_progress(context, id)
-
-        # check parameters
-        aspect_id = body['aspectId']
-        if 'numberOfSteps' not in body:
-            # set default value (1) defined by SOL specification for
-            # the convenience of the following methods.
-            body['numberOfSteps'] = 1
-
-        scale_level = self._get_current_scale_level(inst, aspect_id)
-        max_scale_level = self._get_max_scale_level(inst, aspect_id)
-        if scale_level is None or max_scale_level is None:
-            raise sol_ex.InvalidScaleAspectId(aspect_id=aspect_id)
-
-        num_steps = body['numberOfSteps']
-        if body['type'] == 'SCALE_IN':
-            num_steps *= -1
-        scale_level += num_steps
-        if scale_level < 0 or scale_level > max_scale_level:
-            raise sol_ex.InvalidScaleNumberOfSteps(
-                num_steps=body['numberOfSteps'])
-
-        lcmocc = self._new_lcmocc(id, v2fields.LcmOperationType.SCALE,
-                                  body)
-        lcmocc.create(context)
-
-        self.conductor_rpc.start_lcm_op(context, lcmocc.id)
+        lcmocc = vnflcm_utils.scale(context, id, body, inst=inst)
 
         location = lcmocc_utils.lcmocc_href(lcmocc.id, self.endpoint)
 
         return sol_wsgi.SolResponse(202, None, location=location)
 
     @validator.schema(schema.HealVnfRequest_V200, '2.0.0')
-    @coordinate.lock_vnf_instance('{id}')
     def heal(self, request, id, body):
         context = request.context
         inst = inst_utils.get_inst(context, id)
+        if config.CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(POLICY_NAME.format('heal'),
+                        target=self._get_policy_target(inst))
 
-        if inst.instantiationState != 'INSTANTIATED':
-            raise sol_ex.VnfInstanceIsNotInstantiated(inst_id=id)
-
-        lcmocc_utils.check_lcmocc_in_progress(context, id)
-
-        # check parameter for later use
-        is_all = body.get('additionalParams', {}).get('all', False)
-        if not isinstance(is_all, bool):
-            raise sol_ex.SolValidationError(
-                detail="additionalParams['all'] must be bool.")
-
-        if 'vnfcInstanceId' in body:
-            inst_info = inst.instantiatedVnfInfo
-            vnfc_id = []
-            if inst_info.obj_attr_is_set('vnfcInfo'):
-                vnfc_id = [vnfc.id for vnfc in inst_info.vnfcInfo]
-            for req_vnfc_id in body['vnfcInstanceId']:
-                if req_vnfc_id not in vnfc_id:
-                    raise sol_ex.SolValidationError(
-                        detail="vnfcInstanceId(%s) does not exist."
-                        % req_vnfc_id)
-
-        lcmocc = self._new_lcmocc(id, v2fields.LcmOperationType.HEAL, body)
-        lcmocc.create(context)
-
-        self.conductor_rpc.start_lcm_op(context, lcmocc.id)
+        lcmocc = vnflcm_utils.heal(context, id, body, inst=inst)
 
         location = lcmocc_utils.lcmocc_href(lcmocc.id, self.endpoint)
 
@@ -363,13 +350,16 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
     def change_ext_conn(self, request, id, body):
         context = request.context
         inst = inst_utils.get_inst(context, id)
+        if config.CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(POLICY_NAME.format('change_ext_conn'),
+                        target=self._get_policy_target(inst))
 
         if inst.instantiationState != 'INSTANTIATED':
             raise sol_ex.VnfInstanceIsNotInstantiated(inst_id=id)
 
         lcmocc_utils.check_lcmocc_in_progress(context, id)
 
-        lcmocc = self._new_lcmocc(
+        lcmocc = vnflcm_utils.new_lcmocc(
             id, v2fields.LcmOperationType.CHANGE_EXT_CONN, body)
         lcmocc.create(context)
 
@@ -379,11 +369,51 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
 
         return sol_wsgi.SolResponse(202, None, location=location)
 
+    def _check_vdu_params(self, inst, vdu_params, vim_type, new_script,
+            old_script):
+        inst_vdu_ids = []
+        if inst.instantiatedVnfInfo.obj_attr_is_set('vnfcResourceInfo'):
+            inst_vdu_ids = [inst_vnfc.vduId
+                for inst_vnfc in inst.instantiatedVnfInfo.vnfcResourceInfo]
+
+        for vdu_param in vdu_params:
+            if 'vdu_id' not in vdu_param:
+                raise sol_ex.SolValidationError(
+                    detail="If you set vdu_params in additionalParams, you"
+                           "must set vdu_id for each element.")
+            if vdu_param['vdu_id'] not in inst_vdu_ids:
+                raise sol_ex.SolValidationError(
+                    detail="vdu_id '%s' does not exist." % vdu_param['vdu_id'])
+
+            def _check_vnfc_param(attr):
+                for key in ['cp_name', 'username', 'password']:
+                    if key not in vdu_param[attr]:
+                        raise sol_ex.SolValidationError(
+                            detail=f"If you set {attr} in "
+                                   f"vdu_param, you must set"
+                                   f" 'cp_name', 'username' and"
+                                   f" 'password'")
+
+            if vim_type == 'ETSINFV.OPENSTACK_KEYSTONE.V_3' and new_script:
+                if 'new_vnfc_param' not in vdu_param:
+                    raise sol_ex.SolValidationError(
+                        detail="'new_vnfc_param' must exist in vdu_param")
+                _check_vnfc_param('new_vnfc_param')
+
+            if vim_type == 'ETSINFV.OPENSTACK_KEYSTONE.V_3' and old_script:
+                if 'old_vnfc_param' not in vdu_param:
+                    raise sol_ex.SolValidationError(
+                        detail="'old_vnfc_param' must exist in vdu_param")
+                _check_vnfc_param('old_vnfc_param')
+
     @validator.schema(schema.ChangeCurrentVnfPkgRequest_V200, '2.0.0')
     @coordinate.lock_vnf_instance('{id}')
     def change_vnfpkg(self, request, id, body):
         context = request.context
         inst = inst_utils.get_inst(context, id)
+        if config.CONF.oslo_policy.enhanced_tacker_policy:
+            context.can(POLICY_NAME.format('change_vnfpkg'),
+                        target=self._get_policy_target(inst))
         vnfd_id = body['vnfdId']
 
         if inst.instantiationState != 'INSTANTIATED':
@@ -404,27 +434,21 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
         upgrade_type = additional_params.get('upgrade_type', '')
         if upgrade_type != 'RollingUpdate':
             raise sol_ex.NotSupportUpgradeType(upgrade_type=upgrade_type)
-        if additional_params.get('vdu_params'):
-            vdu_ids = [vdu.get('vdu_id') for vdu in
-                       additional_params.get('vdu_params')]
-            if None in vdu_ids:
-                raise sol_ex.SolValidationError(
-                    detail="If you set vdu_params in additionalParams, you"
-                           "must set vduId for each element.")
-            for vdu in additional_params.get('vdu_params'):
-                for attr in ['old_vnfc_param', 'new_vnfc_param']:
-                    if vdu.get(attr):
-                        vdu_keys = vdu.get(attr).keys()
-                        if set(vdu_keys).difference(set(
-                                ['cp_name', 'username', 'password'])):
-                            raise sol_ex.SolValidationError(
-                                detail=f"If you set {attr} in "
-                                       f"additionalParams, you must set"
-                                       f" 'cp_name', 'username' and"
-                                       f" 'password'")
+        vim_info = inst_utils.select_vim_info(inst.vimConnectionInfo)
+        vdu_params = additional_params.get('vdu_params')
+        if vdu_params is None:
+            raise sol_ex.SolValidationError(
+                detail="'vdu_params' must exist in additionalParams")
+        self._check_vdu_params(inst, vdu_params, vim_info.vimType,
+            additional_params.get('lcm-operation-coordinate-new-vnf'),
+            additional_params.get('lcm-operation-coordinate-old-vnf'))
+        if (vim_info.vimType == 'ETSINFV.KUBERNETES.V_1' and
+                not additional_params.get('lcm-kubernetes-def-files')):
+            raise sol_ex.SolValidationError(
+                detail="'lcm-kubernetes-def-files' must be specified")
 
-        lcmocc = self._new_lcmocc(id, v2fields.LcmOperationType.CHANGE_VNFPKG,
-                                  body)
+        lcmocc = vnflcm_utils.new_lcmocc(
+            id, v2fields.LcmOperationType.CHANGE_VNFPKG, body)
 
         lcmocc.create(context)
 
@@ -451,42 +475,10 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
 
         auth_req = body.get('authentication')
         if auth_req:
-            auth = objects.SubscriptionAuthentication(
-                authType=auth_req['authType']
-            )
-            if 'BASIC' in auth.authType:
-                basic_req = auth_req.get('paramsBasic')
-                if basic_req is None:
-                    msg = "ParmasBasic must be specified."
-                    raise sol_ex.InvalidSubscription(sol_detail=msg)
-                auth.paramsBasic = (
-                    objects.SubscriptionAuthentication_ParamsBasic(
-                        userName=basic_req.get('userName'),
-                        password=basic_req.get('password')
-                    )
-                )
-
-            if 'OAUTH2_CLIENT_CREDENTIALS' in auth.authType:
-                oauth2_req = auth_req.get('paramsOauth2ClientCredentials')
-                if oauth2_req is None:
-                    msg = "paramsOauth2ClientCredentials must be specified."
-                    raise sol_ex.InvalidSubscription(sol_detail=msg)
-                auth.paramsOauth2ClientCredentials = (
-                    objects.SubscriptionAuthentication_ParamsOauth2(
-                        clientId=oauth2_req.get('clientId'),
-                        clientPassword=oauth2_req.get('clientPassword'),
-                        tokenEndpoint=oauth2_req.get('tokenEndpoint')
-                    )
-                )
-
-            if 'TLS_CERT' in auth.authType:
-                msg = "'TLS_CERT' is not supported at the moment."
-                raise sol_ex.InvalidSubscription(sol_detail=msg)
-
-            subsc.authentication = auth
+            subsc.authentication = subsc_utils.check_http_client_auth(auth_req)
 
         if CONF.v2_nfvo.test_callback_uri:
-            subsc_utils.test_notification(subsc)
+            common_script_utils.test_notification(subsc)
 
         subsc.create(context)
 
@@ -502,11 +494,14 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
         else:
             filters = None
 
-        subscs = subsc_utils.get_subsc_all(request.context)
+        pager = self._subsc_view.parse_pager(request)
 
-        resp_body = self._subsc_view.detail_list(subscs, filters)
+        subscs = subsc_utils.get_subsc_all(request.context,
+                                           marker=pager.marker)
 
-        return sol_wsgi.SolResponse(200, resp_body)
+        resp_body = self._subsc_view.detail_list(subscs, filters, pager)
+
+        return sol_wsgi.SolResponse(200, resp_body, link=pager.get_link())
 
     def subscription_show(self, request, id):
         subsc = subsc_utils.get_subsc(request.context, id)
@@ -532,11 +527,15 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
 
         selector = self._lcmocc_view.parse_selector(request.GET)
 
-        lcmoccs = lcmocc_utils.get_lcmocc_all(request.context)
+        pager = self._lcmocc_view.parse_pager(request)
 
-        resp_body = self._lcmocc_view.detail_list(lcmoccs, filters, selector)
+        lcmoccs = lcmocc_utils.get_lcmocc_all(request.context,
+                                              marker=pager.marker)
 
-        return sol_wsgi.SolResponse(200, resp_body)
+        resp_body = self._lcmocc_view.detail_list(lcmoccs, filters,
+                                                  selector, pager)
+
+        return sol_wsgi.SolResponse(200, resp_body, link=pager.get_link())
 
     def lcm_op_occ_show(self, request, id):
         lcmocc = lcmocc_utils.get_lcmocc(request.context, id)
@@ -569,9 +568,6 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
             inst = inst_utils.get_inst(context, lcmocc.vnfInstanceId)
             vim_infos = inst.vimConnectionInfo
         vim_info = inst_utils.select_vim_info(vim_infos)
-        if lcmocc.operation != 'CHANGE_VNFPKG' and (
-                vim_info.vimType == 'kubernetes'):
-            raise sol_ex.NotSupportOperationType
 
         self.conductor_rpc.retry_lcm_op(context, lcmocc.id)
 
@@ -601,9 +597,6 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
             inst = inst_utils.get_inst(context, lcmocc.vnfInstanceId)
             vim_infos = inst.vimConnectionInfo
         vim_info = inst_utils.select_vim_info(vim_infos)
-        if lcmocc.operation != 'CHANGE_VNFPKG' and (
-                vim_info.vimType == 'kubernetes'):
-            raise sol_ex.NotSupportOperationType
 
         self.conductor_rpc.rollback_lcm_op(context, lcmocc.id)
 
@@ -652,3 +645,21 @@ class VnfLcmControllerV2(sol_wsgi.SolAPIController):
         lcmocc.delete(context)
 
         return sol_wsgi.SolResponse(204, None)
+
+    def supported_api_versions(self, action):
+        if action == 'api_versions':
+            # support all versions and it is OK there is no Version header.
+            return None
+        else:
+            return api_version.v2_versions
+
+    def allowed_content_types(self, action):
+        if action == 'update':
+            # Content-Type of Modify request shall be
+            # 'application/mergepatch+json' according to SOL spec.
+            # But 'application/json' and 'text/plain' is OK for backward
+            # compatibility.
+            return ['application/mergepatch+json', 'application/json',
+                    'text/plain']
+        else:
+            return ['application/json', 'text/plain']
