@@ -16,12 +16,14 @@
 from oslo_log import log as logging
 from oslo_service import loopingcall
 
+from tacker.sol_refactored.common import config
 from tacker.sol_refactored.common import exceptions as sol_ex
 from tacker.sol_refactored.common import http_client
 
 
 LOG = logging.getLogger(__name__)
 
+CONF = config.CONF
 
 CHECK_INTERVAL = 5
 
@@ -29,14 +31,28 @@ CHECK_INTERVAL = 5
 class HeatClient(object):
 
     def __init__(self, vim_info):
-        auth = http_client.KeystonePasswordAuthHandle(
-            auth_url=vim_info.interfaceInfo['endpoint'],
-            username=vim_info.accessInfo['username'],
-            password=vim_info.accessInfo['password'],
-            project_name=vim_info.accessInfo['project'],
-            user_domain_name=vim_info.accessInfo['userDomain'],
-            project_domain_name=vim_info.accessInfo['projectDomain']
-        )
+        if CONF.v2_vnfm.use_oauth2_mtls_for_heat:
+            auth = http_client.OAuth2MtlsAuthHandle(
+                endpoint=None,
+                token_endpoint=vim_info.interfaceInfo['endpoint'],
+                client_id=vim_info.accessInfo['username'],
+                ca_cert=CONF.v2_vnfm.heat_mtls_ca_cert_file,
+                client_cert=CONF.v2_vnfm.heat_mtls_client_cert_file
+            )
+        else:
+            verify = CONF.v2_vnfm.heat_verify_cert
+            if verify and CONF.v2_vnfm.heat_ca_cert_file:
+                verify = CONF.v2_vnfm.heat_ca_cert_file
+            auth = http_client.KeystonePasswordAuthHandle(
+                auth_url=vim_info.interfaceInfo['endpoint'],
+                username=vim_info.accessInfo['username'],
+                password=vim_info.accessInfo['password'],
+                project_name=vim_info.accessInfo['project'],
+                user_domain_name=vim_info.accessInfo['userDomain'],
+                project_domain_name=vim_info.accessInfo['projectDomain'],
+                verify=verify
+            )
+
         self.client = http_client.HttpClient(auth,
                                              service_type='orchestration')
 
@@ -53,7 +69,13 @@ class HeatClient(object):
 
     def update_stack(self, stack_name, fields, wait=True):
         path = f"stacks/{stack_name}"
-        resp, body = self.client.do_request(path, "PATCH",
+        # It was assumed that PATCH is used and therefore 'fields'
+        # contains only update parts and 'existing' is not used.
+        # Now full replacing 'fields' is supported and it is indicated
+        # by 'existing' is False.  if 'existing' is specified and
+        # it is False, PUT is used.
+        method = "PATCH" if fields.pop('existing', True) else "PUT"
+        resp, body = self.client.do_request(path, method,
                  expected_status=[202], body=fields)
 
         if wait:
@@ -107,7 +129,7 @@ class HeatClient(object):
                 LOG.info("%s %s done.", operation, stack_name.split('/')[0])
                 raise loopingcall.LoopingCallDone()
             elif status in failed_status:
-                LOG.error("% %s failed.", operation, stack_name.split('/')[0])
+                LOG.error("%s %s failed.", operation, stack_name.split('/')[0])
                 sol_title = "%s failed" % operation
                 raise sol_ex.StackOperationFailed(sol_title=sol_title,
                                                   sol_detail=status_reason)
@@ -200,12 +222,30 @@ def get_port_reses(heat_reses):
 
 
 def get_stack_name(inst, stack_id=None):
+    """Return different stack name for different operations
+
+    * A new "stack_id" will be generated after the instantiate and heal_all
+      perform the "create_stack" operation. When make instantiated_vnf_info,
+      the "stack_id" will be passed in as an input parameter.
+      :return: vnf-{inst.id}/{stack_id}
+    * In other lifecycle operations, "stack_id" is stored in the "metadata" of
+      "instantiatedVnfInfo".
+      :return: vnf-{inst.id}/{inst.instantiatedVnfInfo.metadata['stack_id']}
+    * In the instantiate and instantiate_rollback operations, the "stack_id"
+      is not yet known.
+      :return: vnf-{inst.id}
+    """
     stack_name = f"vnf-{inst.id}"
-    if inst.obj_attr_is_set('instantiatedVnfInfo') and not stack_id:
-        return f"{stack_name}/{inst.instantiatedVnfInfo.metadata['stack_id']}"
     if stack_id:
         return f"{stack_name}/{stack_id}"
-    return stack_name
+    # Add a check for the existence of "metadata" when instantiate again
+    # after terminate
+    elif (inst.obj_attr_is_set('instantiatedVnfInfo') and
+            inst.instantiatedVnfInfo.obj_attr_is_set('metadata') and
+            inst.instantiatedVnfInfo.metadata.get('stack_id')):
+        return f"{stack_name}/{inst.instantiatedVnfInfo.metadata['stack_id']}"
+    else:
+        return stack_name
 
 
 def get_resource_stack_id(heat_res):
