@@ -18,6 +18,7 @@ import collections
 import contextlib
 import datetime
 
+from oslo_db.sqlalchemy import utils
 from oslo_log import log as logging
 import oslo_messaging as messaging
 from oslo_serialization import jsonutils
@@ -26,6 +27,7 @@ from oslo_versionedobjects import base as ovoo_base
 from oslo_versionedobjects import exception as ovoo_exc
 
 from tacker.db import api as db_api
+from tacker.sol_refactored.common import exceptions as sol_ex
 from tacker.sol_refactored.db.sqlalchemy import models
 from tacker.sol_refactored import objects
 from tacker.sol_refactored.objects import fields as obj_fields
@@ -376,9 +378,15 @@ class TackerPersistentObject(TackerObject):
 
     @classmethod
     @db_api.context_manager.reader
-    def get_all(cls, context):
+    def get_all(cls, context, marker=None):
         model_cls = getattr(models, cls.__name__)
         query = context.session.query(model_cls)
+        if marker is not None:
+            db_obj = query.filter(model_cls.id == marker).one_or_none()
+            if db_obj is None:
+                raise sol_ex.InvalidPagingMarker(marker=marker)
+            query = utils.paginate_query(query, model_cls, None, ['id'],
+                                         marker=db_obj)
         result = query.all()
         return [cls.from_db_obj(item) for item in result]
 
