@@ -13,7 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import mock
+from unittest import mock
+
 from oslo_utils import uuidutils
 
 from tacker.common import exceptions
@@ -46,6 +47,7 @@ class TestVnfPackage(SqlTestCase):
         self.assertEqual('CREATED', vnfpkgm.onboarding_state)
         self.assertEqual('NOT_IN_USE', vnfpkgm.usage_state)
         self.assertEqual('DISABLED', vnfpkgm.operational_state)
+        self.assertEqual(0, vnfpkgm.size)
 
     def test_create_without_user_define_data(self):
         vnf_pack = fakes.vnf_package_data
@@ -85,7 +87,8 @@ class TestVnfPackage(SqlTestCase):
                                                self.vnf_package.id,
                                                expected_attrs=None)
         self.compare_obj(self.vnf_package, vnfpkgm,
-                         allow_missing=['vnf_deployment_flavours'])
+                         allow_missing=['vnf_deployment_flavours',
+                                        'vnf_artifacts'])
 
     def test_get_by_id_with_no_existing_id(self):
         self.assertRaises(
@@ -94,9 +97,15 @@ class TestVnfPackage(SqlTestCase):
             uuidsentinel.invalid_uuid)
 
     def test_create_with_id(self):
-        vnf_obj = {'id': uuidsentinel.uuid}
-        vnf_pack = objects.VnfPackage(context=self.context, **vnf_obj)
-        self.assertRaises(exceptions.ObjectActionError, vnf_pack.create)
+        vnfpkgm = objects.VnfPackage(context=self.context,
+                **fakes.vnf_package_data)
+        vnfpkgm['id'] = uuidsentinel.uuid
+        vnfpkgm.create()
+        self.assertTrue(vnfpkgm.id)
+        self.assertEqual('CREATED', vnfpkgm.onboarding_state)
+        self.assertEqual('NOT_IN_USE', vnfpkgm.usage_state)
+        self.assertEqual('DISABLED', vnfpkgm.operational_state)
+        self.assertEqual(0, vnfpkgm.size)
 
     def test_save(self):
         self.vnf_package.onboarding_state = 'ONBOARDED'
@@ -122,7 +131,7 @@ class TestVnfPackage(SqlTestCase):
     def test_get_all(self):
         result = objects.VnfPackagesList.get_all(self.context,
                                                  expected_attrs=None)
-        self.assertTrue(result.objects, list)
+        self.assertIsInstance(result.objects, list)
         self.assertTrue(result.objects)
 
     def test_get_by_id_with_flavours(self):
@@ -171,7 +180,17 @@ class TestVnfPackage(SqlTestCase):
             vnf_deployment_flavour_obj.obj_load_attr, 'algorithm')
 
     def test_vnf_package_list_by_filter(self):
-        filters = {'onboarding_state': 'CREATED'}
+        filters = {'field': 'onboarding_state', 'model': 'VnfPackage',
+                   'value': 'CREATED',
+                   'op': '=='}
         vnfpkgm_list = objects.VnfPackagesList.get_by_filters(
-            self.context, **filters)
+            self.context, filters=filters)
         self.assertEqual(1, len(vnfpkgm_list))
+
+    def test_obj_make_compatible(self):
+        data = {'id': self.vnf_package.id}
+        vnf_package_obj = objects.VnfPackage(context=self.context, **data)
+        fake_vnf_package_obj = objects.VnfPackage(context=self.context, **data)
+        obj_primitive = fake_vnf_package_obj.obj_to_primitive('1.0')
+        obj = vnf_package_obj.obj_from_primitive(obj_primitive)
+        self.assertIn('size', obj.fields)
