@@ -27,8 +27,8 @@ def _get_template(name):
     filename = os.path.join(
         os.path.dirname(os.path.abspath(__file__)),
         "../infra_drivers/openstack/data/", name)
-    f = codecs.open(filename, encoding='utf-8', errors='strict')
-    return f.read()
+    with codecs.open(filename, encoding='utf-8', errors='strict') as f:
+        return f.read()
 
 
 class TestToscaUtils(testtools.TestCase):
@@ -91,22 +91,22 @@ class TestToscaUtils(testtools.TestCase):
                     nt.type_definition.is_derived_from(toscautils.PLACEMENT)):
                 invalidNodes += 1
 
-            if nt.type in toscautils.delpropmap.keys():
+            if nt.type in toscautils.delpropmap:
                 for prop in toscautils.delpropmap[nt.type]:
                     for p in nt.get_properties_objects():
                         if prop == p.name:
                             deletedProperties += 1
 
             if nt.type in toscautils.convert_prop_values:
-                for prop in toscautils.convert_prop_values[nt.type].keys():
+                for prop in toscautils.convert_prop_values[nt.type]:
                     convertmap = toscautils.convert_prop_values[nt.type][prop]
                     for p in nt.get_properties_objects():
                         if (prop == p.name and
-                                p.value in convertmap.keys()):
+                                p.value in convertmap):
                             convertedValues += 1
 
             if nt.type in toscautils.convert_prop:
-                for prop in toscautils.convert_prop[nt.type].keys():
+                for prop in toscautils.convert_prop[nt.type]:
                     for p in nt.get_properties_objects():
                         if prop == p.name:
                             convertedProperties += 1
@@ -285,7 +285,7 @@ class TestToscaUtils(testtools.TestCase):
         expected_dict = {
             'volumes': {
                 'VB1': {
-                    'image': 'cirros-0.4.0-x86_64-disk',
+                    'image': 'cirros-0.5.2-x86_64-disk',
                     'size': '1'
                 }
             },
@@ -298,3 +298,23 @@ class TestToscaUtils(testtools.TestCase):
         }
         volume_details = toscautils.get_block_storage_details(vnfd_dict)
         self.assertEqual(expected_dict, volume_details)
+
+    def test_get_block_storage_details_volume_id(self):
+        tosca_vol = _get_template(
+            'test-tosca-vnfd-existing-block-storage.yaml')
+        vnfd_dict = yaml.safe_load(tosca_vol)
+        expected_dict = {
+            'volumes': {
+                'VB1': {
+                    'volume_id': 'my_vol'
+                }
+            },
+            'volume_attachments': {
+                'CB1': {
+                    'instance_uuid': {'get_resource': 'VDU1'},
+                    'mountpoint': '/dev/vdb',
+                    'volume_id': {'get_param': 'my_vol'}}
+            }
+        }
+        volume_details = toscautils.get_block_storage_details(vnfd_dict)
+        self.assertEqual(expected_dict, volume_details)
