@@ -14,8 +14,9 @@
 import copy
 
 from oslo_log import log as logging
+from oslo_serialization import jsonutils
 from tacker.common.utils import MemoryUnit
-
+from tacker.tosca import utils as tosca_utils
 
 """Define util functions that can be used in UserData.
 
@@ -106,7 +107,7 @@ def create_initial_param_dict(base_hot_dict):
                             if param_list[2] not in resource_info:
                                 resource_info[param_list[2]] = {}
 
-    LOG.info('initial_param_dict: %s', initial_param_dict)
+    LOG.debug('initial_param_dict: %s', initial_param_dict)
     return initial_param_dict
 
 
@@ -177,7 +178,7 @@ def create_initial_param_server_port_dict(base_hot_dict):
                             if param_list[2] not in resource_info:
                                 resource_info[param_list[2]] = {}
 
-    LOG.info('initial_param_dict: %s', initial_param_dict)
+    LOG.debug('initial_param_dict: %s', initial_param_dict)
     return initial_param_dict
 
 
@@ -209,7 +210,7 @@ def create_final_param_dict(param_dict, vdu_flavor_dict,
                 for fixed_ip in cpd_vl_dict.get(target_cp).get("fixed_ips"):
                     cps[target_cp]['fixed_ips'].append(fixed_ip)
 
-    LOG.info('final_param_dict: %s', final_param_dict)
+    LOG.debug('final_param_dict: %s', final_param_dict)
     return final_param_dict
 
 
@@ -243,7 +244,7 @@ def create_vdu_flavor_dict(vnfd_dict):
                         val[0]['size_of_storage'], 'GiB')
             vdu_flavor_dict[vdu_name] = flavor_dict
 
-    LOG.info('vdu_flavor_dict: %s', vdu_flavor_dict)
+    LOG.debug('vdu_flavor_dict: %s', vdu_flavor_dict)
     return vdu_flavor_dict
 
 
@@ -258,7 +259,7 @@ def create_vdu_image_dict(grant_info):
         for vnf_resource in resources:
             vdu_image_dict[vdu_name] = vnf_resource.resource_identifier
 
-    LOG.info('vdu_image_dict: %s', vdu_image_dict)
+    LOG.debug('vdu_image_dict: %s', vdu_image_dict)
     return vdu_image_dict
 
 
@@ -284,7 +285,7 @@ def create_cpd_vl_dict(base_hot_dict, inst_req_info):
                     cpd_vl_dict[ext_cp.cpd_id] = vl_uuid
                     break
 
-    LOG.info('cpd_vl_dict: %s', cpd_vl_dict)
+    LOG.debug('cpd_vl_dict: %s', cpd_vl_dict)
     return cpd_vl_dict
 
 
@@ -336,7 +337,7 @@ def create_vdu_flavor_capability_name_dict(vnfd_dict):
                                 "requested_additional"
                                 "_capability_name"]
 
-    LOG.info('vdu_flavor_dict: %s', vdu_flavor_dict)
+    LOG.debug('vdu_flavor_dict: %s', vdu_flavor_dict)
     return vdu_flavor_dict
 
 
@@ -358,7 +359,7 @@ def create_sw_image_dict(vnfd_dict):
             if 'name' in sw_image_data_props.keys():
                 sw_image_data[vdu_name] = sw_image_data_props['name']
 
-    LOG.info('sw_image_data: %s', sw_image_data)
+    LOG.debug('sw_image_data: %s', sw_image_data)
     return sw_image_data
 
 
@@ -381,7 +382,7 @@ def create_network_dict(inst_req_info, param_dict):
                 cp_data[ext_cp.cpd_id]["fixed_ips"] =\
                     _create_fixed_ips_list(ext_cp)
 
-    LOG.info('cp_data: %s', cp_data)
+    LOG.debug('cp_data: %s', cp_data)
     return cp_data
 
 
@@ -412,3 +413,85 @@ def _create_fixed_ips_list(ext_cp):
                     fixed_ips_lst.append(fixed_ips)
 
     return fixed_ips_lst
+
+
+def _create_scale_group_dict(base_hot_dict, vnfd_dict, inst_req_info):
+
+    base_hot = base_hot_dict['heat_template']
+    LOG.debug("base_hot: %s", base_hot)
+
+    scaling_group_dict = {}
+    for name, rsc in base_hot.get('resources').items():
+        if rsc['type'] == 'OS::Heat::AutoScalingGroup':
+            key_name = name.replace('_group', '')
+            scaling_group_dict[key_name] = name
+    LOG.debug("scaling_group_dict: %s", scaling_group_dict)
+
+    if scaling_group_dict:
+        vnf = {'attributes': {'scaling_group_names':
+            jsonutils.dump_as_bytes(scaling_group_dict)}}
+        scale_group_dict = tosca_utils.get_scale_group(
+            vnf, vnfd_dict, inst_req_info)
+        LOG.debug("scale_group_dict: %s", scale_group_dict)
+        return scale_group_dict
+    else:
+        LOG.debug("no scale_group_dict")
+        return {}
+
+
+def create_desired_capacity_dict(base_hot_dict, vnfd_dict, inst_req_info):
+    """Create a dict containing information about desired capacity.
+
+    :param base_hot_dict: dict(Base HOT dict format)
+    :param vnfd_dict: dict(VNFD dict format)
+    :param inst_req_info: dict(Instantiation request information format)
+    :return: dict(Scaling aspect name, Desired capacity value)
+    """
+    scale_group_dict = _create_scale_group_dict(
+        base_hot_dict, vnfd_dict, inst_req_info)
+
+    param_dict = {}
+    if scale_group_dict.get('scaleGroupDict'):
+        for name, value in scale_group_dict['scaleGroupDict'].items():
+            param_dict[name] = value['default']
+
+    LOG.debug("desired_capacity dict: %s", param_dict)
+    return param_dict
+
+
+def _calc_desired_capacity(inst_vnf_info, name, value):
+
+    for scale_status in inst_vnf_info.scale_status:
+        if scale_status.aspect_id == name:
+            LOG.debug("scale_level of %s: %d",
+                 name, scale_status.scale_level)
+            increase = value['num'] * scale_status.scale_level
+            desired_capacity = value['initialNum'] + increase
+            LOG.debug("desired_capacity: %d", desired_capacity)
+            return desired_capacity
+
+    LOG.debug("scale_level of %s: None", name)
+    return None
+
+
+def get_desired_capacity_dict(base_hot_dict, vnfd_dict, inst_vnf_info):
+    """Get a dict containing information about desired capacity.
+
+    :param base_hot_dict: dict(Base HOT dict format)
+    :param vnfd_dict: dict(VNFD dict format)
+    :param inst_vnf_info: dict(Instantiated VNF Info dict format)
+    :return: dict(Scaling aspect name, Desired capacity value)
+    """
+    scale_group_dict = _create_scale_group_dict(
+        base_hot_dict, vnfd_dict, {})
+
+    param_dict = {}
+    if scale_group_dict.get('scaleGroupDict'):
+        for name, value in scale_group_dict['scaleGroupDict'].items():
+            desired_capacity = _calc_desired_capacity(
+                inst_vnf_info, name, value)
+            if desired_capacity is not None:
+                param_dict[name] = desired_capacity
+
+    LOG.debug("desired_capacity dict: %s", param_dict)
+    return param_dict
